package qcow-types

  1. Overview
  2. Docs

Module Qcow_cluster_map.AvailableSource

Clusters which are available for reallocation

get t query the current contents of the set

remove t less removes less from the set

Sourceval mem : t -> Qcow_types.Cluster.t -> bool

mem t cluster is true if cluster is in t