package qcow-types

  1. Overview
  2. Docs

Module Qcow_metadata.RefcountsSource

Sourcetype t

A cluster full of 16bit refcounts

Sourceval of_contents : contents -> t

Interpret the given cluster as a refcount cluster

Sourceval get : t -> int -> int

get t n return the nth refcount within t

Sourceval set : t -> int -> int -> unit

set t n v set the nth refcount within t to v